#!/bin/env python3

import argparse
import os
import re
import subprocess
import sys
import tempfile

# Path relative to this script
uncrustify_cfg = 'tools/uncrustify.cfg'

def run_diff(sha):
    proc = subprocess.Popen(["git", "diff", "-U0", "--function-context", sha, "HEAD"], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    files = proc.stdout.read().strip().decode('utf-8')
    return files.split('\n')

def find_chunks(diff):
    file_entry_re = re.compile('^\+\+\+ b/(.*)$')
    diff_chunk_re = re.compile('^@@ -\d+,\d+ \+(\d+),(\d+)')
    file = None
    chunks = []

    for line in diff:
        match = file_entry_re.match(line)
        if match:
            file = match.group(1)

        match = diff_chunk_re.match(line)
        if match:
            start = int(match.group(1))
            len = int(match.group(2))
            end = start + len

            if len > 0 and (file.endswith('.c') or file.endswith('.h') or file.endswith('.vala')):
                chunks.append({ 'file': file, 'start': start, 'end': end })

    return chunks

def reformat_chunks(chunks, rewrite):
    # Creates temp file with INDENT-ON/OFF comments
    def create_temp_file(file, start, end):
        with open(file) as f:
            tmp = tempfile.NamedTemporaryFile()
            tmp.write(b'/** *INDENT-OFF* **/\n')
            for i, line in enumerate(f):
                if i == start - 2:
                    tmp.write(b'/** *INDENT-ON* **/\n')

                tmp.write(bytes(line, 'utf-8'))

                if i == end - 2:
                    tmp.write(b'/** *INDENT-OFF* **/\n')

            tmp.seek(0)

        return tmp

    # Removes uncrustify INDENT-ON/OFF helper comments
    def remove_indent_comments(output):
        tmp = tempfile.NamedTemporaryFile()

        for line in output:
            if line != b'/** *INDENT-OFF* **/\n' and line != b'/** *INDENT-ON* **/\n':
                tmp.write(line)

        tmp.seek(0)

        return tmp

    changed = None

    for chunk in chunks:
        # Add INDENT-ON/OFF comments
        tmp = create_temp_file(chunk['file'], chunk['start'], chunk['end'])

        # uncrustify chunk
        proc = subprocess.Popen(["uncrustify", "-c", uncrustify_cfg, "-f", tmp.name], stdout=subprocess.PIPE)
        reindented = proc.stdout.readlines()
        proc.wait()
        if proc.returncode != 0:
            continue

        tmp.close()

        # Remove INDENT-ON/OFF comments
        formatted = remove_indent_comments(reindented)

        if dry_run is True:
            # Show changes
            proc = subprocess.Popen(["diff", "-up", "--color=always", chunk['file'], formatted.name], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            diff = proc.stdout.read().decode('utf-8')
            if diff != '':
                output = re.sub('\t', '↦\t', diff)
                print(output)
                changed = True
        else:
            # Apply changes
            diff = subprocess.Popen(["diff", "-up", chunk['file'], formatted.name], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            patch = subprocess.Popen(["patch", chunk['file']], stdin=diff.stdout)
            diff.stdout.close()
            patch.communicate()

        formatted.close()

    return changed


parser = argparse.ArgumentParser(description='Check code style.')
parser.add_argument('--sha', metavar='SHA', type=str,
                    help='SHA for the commit to compare HEAD with')
parser.add_argument('--dry-run', '-d', type=bool,
                    action=argparse.BooleanOptionalAction,
                    help='Only print changes to stdout, do not change code')
parser.add_argument('--rewrite', '-r', type=bool,
                    action=argparse.BooleanOptionalAction,
                    help='Whether to amend the result to the last commit (e.g. \'git rebase --exec "%(prog)s -r"\')')

# Change CWD to script location, necessary for always locating the configuration file
os.chdir(os.path.dirname(os.path.abspath(sys.argv[0])))

args = parser.parse_args()
sha = args.sha or 'HEAD^'
rewrite = args.rewrite
dry_run = args.dry_run

diff = run_diff(sha)
chunks = find_chunks(diff)
changed = reformat_chunks(chunks, rewrite)

if dry_run is not True and rewrite is True:
    proc = subprocess.Popen(["git", "commit", "--all", "--amend", "-C", "HEAD"], stdout=subprocess.DEVNULL)
    os._exit(0)
elif dry_run is True and changed is True:
    print ("\nIssue the following command in your local tree to apply the suggested changes (needs uncrustify installed):\n\n $ git rebase origin/main --exec \"./check-style.py -r\" \n")
    os._exit(-1)

os._exit(0)



#!/usr/bin/python
#
# release-wrangler.py - very basic release system, primarily for
# Metacity, might be useful for others. In very early stages of
# development.
#
# Copyright (C) 2008 Thomas Thurman
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import posixpath
import re
import sys
import commands
import time
import commands

def report_error(message):
  print message
  sys.exit(255)

def get_up_to_date():
  "First step is always to get up to date."
  os.system("svn up")

# yes, I know this is MY username. I will come back and fix it
# later, but for now there is a lot else to do. FIXME
your_username = 'Thomas Thurman  <tthurman@gnome.org>'

def changelog_and_checkin(filename, message):
  changelog = open('ChangeLog.tmp', 'w')
  changelog.write('%s  %s\n\n        * %s: %s\n\n' % (
    time.strftime('%Y-%m-%d',time.gmtime()),
    your_username,
    filename,
    message))

  for line in open('ChangeLog').readlines():
    changelog.write(line)

  changelog.close()
  os.rename('ChangeLog.tmp', 'ChangeLog')

  if os.system('svn commit -m "%s"' % (message.replace('"','\\"')))!=0:
    report_error("Could not commit; bailing.")

def check_we_are_up_to_date():
  changed = []
  for line in commands.getoutput('/usr/bin/svn status').split('\n'):
    if line!='' and (line[0]=='C' or line[0]=='M'):
      if line.find('release-wrangler.py')==-1 and line.find('ChangeLog')==-1:
        # we should be insensitive to changes in this script itself
        # to avoid chicken-and-egg problems
        changed.append(line[1:].lstrip())

  if changed:
    report_error('These files are out of date; I can\'t continue until you fix them: ' + \
      ', '.join(changed))

def version_numbers():
  # FIXME: This is all very metacity-specific. Compare fusa, etc
  """Okay, read through configure.in and find who and where we are.

  We also try to figure out where the next micro version number
  will be; some programs (e.g. Metacity) use a custom numbering
  scheme, and if they have a list of numbers on the line before the
  micro version then that will be used. Otherwise we will just
  increment."""

  version = {}
  previous_line = ''
  for line in file("configure.in").readlines():
    product_name = re.search("^AC_INIT\(\[([^\]]*)\]", line)
    if product_name:
      version['name'] = product_name.group(1)

    version_number = re.search("^m4_define\(\[.*_(.*)_version\], \[(\d+)\]", line)

    if version_number:
      version_type = version_number.group(1)
      version_value = int(version_number.group(2))

      version[version_type] = version_value

      if version_type == 'micro':
        group_of_digits = re.search("^\#\s*([0-9, ]+)\n$", previous_line)
        if group_of_digits:
          versions = [int(x) for x in group_of_digits.group(1).split(',')]

          if version_value in versions:
            try:
              version_index = versions.index(version_value)+1

              if versions[version_index] == version['micro']:
                # work around metacity giving "1" twice
                version_index += 1

              version['micro_next'] = versions[version_index]
            except:
              report_error("You gave a list of micro version numbers, but we've used them up!")
          else:
            report_error("You gave a list of micro version numbers, but the current one wasn't in it! Current is %s and your list is %s" % (
              `version_value`, `versions`))

    previous_line = line

  if not 'micro_next' in version:
    version['micro_next'] = version['micro']+1

  version['string'] = '%(major)s.%(minor)s.%(micro)s' % (version)
  version['filename'] = '%(name)s-%(string)s.tar.gz' % (version)

  return version

def check_file_does_not_exist(version):
  if os.access(version['filename'], os.F_OK):
    report_error("Sorry, you already have a file called %s! Please delete it or move it first." % (version['filename']))

def is_date(str):
  return len(str)>4 and str[4]=='-'

def scan_changelog(version):
  changelog = file("ChangeLog").readlines()

  # Find the most recent release.

  release_date = None

  for line in changelog:
    if is_date(line):
      release_date = line[:10]

    if "Post-release bump to" in line:
      changelog = changelog[:changelog.index(line)+1]
      break

  contributors = {}
  thanks = ''
  entries = []

  def assumed_surname(name):
    if name=='': return ''
    # might get more complicated later, but for now...
    return name.split()[-1]

  def assumed_forename(name):
    if name=='': return ''
    return name.split()[0]

  bug_re = re.compile('bug \#?(\d+)', re.IGNORECASE)
  hash_re = re.compile('\#(\d+)')

  for line in changelog:
    if is_date(line):
      line = line[10:].lstrip()
      line = line[:line.find('<')].rstrip()
      contributors[assumed_surname(line)] = line
      entries.append('(%s)' % (assumed_forename(line)))
    else:
      match = bug_re.search(line)
      if not match: match = hash_re.search(line)
      if match:
        entries[-1] += ' (#%s)' % (match.group(1))

  # FIXME: getting complex enough we should be returning a dictionary
  return (contributors, changelog, entries, release_date)

def wordwrap(str, prefix=''):
  "Really simple wordwrap"

  # Ugly hack:
  # We know that all open brackets are preceded by spaces.
  # We don't want to split on these spaces. Therefore:
  str = str.replace(' (','(')

  result = ['']
  for word in str.split():

    if result[-1]=='':
      candidate = prefix + word
    else:
      candidate = '%s %s' % (result[-1], word)

    if len(candidate)>80:
      result.append(prefix+word)
    else:
      result[-1] = candidate

  return '\n'.join(result).replace('(',' (')

def favourite_editor():
  e = os.environ
  if e.has_key('VISUAL'): return e['VISUAL']
  if e.has_key('EDITOR'): return e['EDITOR']
  if os.access('/usr/bin/nano', os.F_OK):
    return '/usr/bin/nano'
  report_error("I can't find an editor for you!")

def edit_news_entry(version):

  # FIXME: still needs a lot of tidying up. Translator stuff especially needs to be
  # factored out into a separate function.

  (contributors, changelog, entries, release_date) = scan_changelog(version)

  contributors_list = contributors.keys()
  contributors_list.sort()
  thanksline = ', '.join([contributors[x] for x in contributors_list])
  thanksline = thanksline.replace(contributors[contributors_list[-1]], 'and '+contributors[contributors_list[-1]])

  thanks = '%s\n%s\n\n' % (version['string'], '='*len(version['string']))
  thanks += wordwrap('Thanks to %s for improvements in this version.' % (thanksline))
  thanks += '\n\n'
  for line in entries:
    thanks += '  - xxx %s\n' % (line)

  # and now pick up the translations.

  translations = {}
  language_re = re.compile('\*\s*(.+)\.po')

  for line in file("po/ChangeLog").readlines():
    match = language_re.search(line)
    if match:
      translations[match.group(1)] = 1
    if is_date(line) and line[:10]<release_date:
      break

  translator_list = translations.keys()
  translator_list.sort()

  last_translator_re = re.compile('Last-Translator:([^<"]*)', re.IGNORECASE)

  def translator_name(language):
    name = 'unknown'

    if ',' in language:
      language = language[:language.find(',')].replace('.po','')

    filename = 'po/%s.po' % (language)

    if not os.access(filename, os.F_OK):
      # Never mind the translator being unknown, we don't even
      # know about the language!
      return 'Mystery translator (%s)'  % (language)

    for line in file(filename).readlines():
      match = last_translator_re.search(line)
      if match:
        name = match.group(1).rstrip().lstrip()
        break

    return "%s (%s)" % (name, language)

  thanks += '\nTranslations\n'
  thanks += wordwrap(', '.join([translator_name(x) for x in translator_list]), '  ')
  thanks += '\n\n'

  changes = '## '+ ' '.join(changelog).replace('\n', '\n## ')

  filename = posixpath.expanduser("~/.release-wrangler-%(name)s-%(string)s.txt" % version)
  tmp = open(filename, 'w')
  tmp.write('## You are releasing %(name)s, version %(major)s.%(minor)s.%(micro)s.\n' % version)
  tmp.write('## The text at the foot of the page is the part of the ChangeLog which\n')
  tmp.write('## has changed since the last release. Please summarise it.\n')
  tmp.write('## Anything preceded by a # is ignored.\n')
  tmp.write(thanks)
  tmp.write(changes)
  tmp.close()

  os.system(favourite_editor()+' +6 %s ' % (filename))
  # FIXME: if they abort, would be useful to abort here too

  # Write it out to NEWS

  version['announcement'] = ''

  news_tmp = open('NEWS.tmp', 'a')
  for line in open(filename, 'r').readlines():
    if line=='' or line[0]!='#':
      news_tmp.write(line)
      version['announcement'] += line

  for line in open('NEWS').readlines():
    news_tmp.write(line)

  news_tmp.close()

  os.rename('NEWS.tmp', 'NEWS')
  changelog_and_checkin('NEWS', '%(major)s.%(minor)s.%(micro)s release.' % (version))

def build_it_all(version):
  "Now build the thing."
  autogen_prefix= '/prefix' # FIXME: this is specific to tthurman's laptop!

  # FIXME: These should use os.system

  if os.spawnl(os.P_WAIT, './autogen.sh', './autogen.sh', '--prefix', autogen_prefix) != 0:
    print 'autogen failed'
    sys.exit(255)
    
  if os.spawnl(os.P_WAIT, '/usr/bin/make', '/usr/bin/make') != 0:
    print 'make failed'
    sys.exit(255)

  if os.spawnl(os.P_WAIT, '/usr/bin/make', '/usr/bin/make', 'install') != 0:
    print 'install failed'
    sys.exit(255)

  if os.spawnl(os.P_WAIT, '/usr/bin/make', '/usr/bin/make', 'distcheck') != 0:
    print 'distcheck failed'
    sys.exit(255)

  if not os.access(version['filename'], os.F_OK):
    print "Sorry, we don't appear to have a file called %s!" % (archive_filename)
    sys.exit(255)

def upload(version):
  # No, we won't have a configuration option to set your name on master.g.o; that's
  # what ~/.ssh/config is for.

  print "Uploading..."
  upload_result = commands.getstatusoutput('scp %s master.gnome.org:' % (version['filename']))

  if upload_result[0]!=0:
    report_error("There appears to have been an uploading problem: %d\n%s\n" % (upload_result[0], upload_result[1]))

def increment_version(version):
  configure_in = file('configure.in.tmp', 'w')
  for line in file('configure.in'):
    if re.search("^m4_define\(\[.*_micro_version\], \[(\d+)\]", line):
      line = line.replace('[%(micro)s]' % version, '[%(micro_next)s]' % version)
    configure_in.write(line)
  
  configure_in.close()
  os.rename('configure.in.tmp', 'configure.in')

  changelog_and_checkin('configure.in', 'Post-release bump to %(major)s.%(minor)s.%(micro_next)s.' % version)

def tag_the_release(version):
  version['ucname'] = version['name'].upper()
  if os.system("svn cp -m release . svn+ssh://svn.gnome.org/svn/%(name)s/tags/%(ucname)s_%(major)s_%(minor)s_%(micro)s" % (version))!=0:
    report_error("Could not tag; bailing.")

def md5s(version):
  return commands.getstatusoutput('ssh master.gnome.org "cd /ftp/pub/GNOME/sources/%(name)s/%(major)s.%(minor)s/;md5sum $(name)s-%(major)s.%(minor)s.%(micro)s.tar*"' % (version))

def main():
  get_up_to_date()
  check_we_are_up_to_date()
  version = version_numbers()
  check_file_does_not_exist(version)
  edit_news_entry(version)
  build_it_all(version)
  tag_the_release(version)
  increment_version(version)
  upload(version)
  print version['announcement']
  print "-- Done --"

if __name__=='__main__':
  main()




#!/usr/bin/env python3

import dbus
import sys
import os
import fcntl
import subprocess
import getpass
import argparse
from collections import OrderedDict
from dbusmock import DBusTestCase
from dbus.mainloop.glib import DBusGMainLoop

DBusGMainLoop(set_as_default=True)


def get_templates_dir():
    return os.path.join(os.path.dirname(__file__), 'dbusmock-templates')

def get_template_path(template_name):
    return os.path.join(get_templates_dir(), template_name + '.py')

def get_subprocess_stdout():
    if os.getenv('META_DBUS_RUNNER_VERBOSE') == '1':
        return sys.stderr
    else:
        return subprocess.DEVNULL;


class MutterDBusTestCase(DBusTestCase):
    @classmethod
    def setUpClass(klass, enable_kvm):
        klass.mocks = OrderedDict()

        print('Starting D-Bus daemons (session & system)...', file=sys.stderr)
        DBusTestCase.setUpClass()
        klass.start_session_bus()
        klass.start_system_bus()

        print('Starting mocked services...', file=sys.stderr)
        (klass.mocks_manager, klass.mock_obj) = klass.start_from_local_template(
            'meta-mocks-manager', {'templates-dir': get_templates_dir()})

        klass.start_from_local_template('localed')

        klass.system_bus_con = klass.get_dbus(system_bus=True)
        klass.session_bus_con = klass.get_dbus(system_bus=False)

        klass.init_logind(enable_kvm)

        if klass.session_bus_con.name_has_owner('org.gnome.Mutter.DisplayConfig'):
            raise Exception(
                'org.gnome.Mutter.DisplayConfig already has owner on the session bus, bailing')

    @classmethod
    def tearDownClass(klass):
        klass.mock_obj.Cleanup()

        for (mock_server, mock_obj) in reversed(klass.mocks.values()):
            mock_server.terminate()
            mock_server.wait()

        DBusTestCase.tearDownClass()

    @classmethod
    def start_from_template(klass, template, params={}):
        mock_server, mock_obj = \
            klass.spawn_server_template(template,
                                        params,
                                        get_subprocess_stdout())

        mocks = (mock_server, mock_obj)
        assert klass.mocks.setdefault(template, mocks) == mocks
        return mocks

    @classmethod
    def start_from_local_template(klass, template_file_name, params={}):
        template = get_template_path(template_file_name)
        return klass.start_from_template(template, params)

    @classmethod
    def start_from_template_managed(klass, template):
        klass.mock_obj.StartFromTemplate(template)

    @classmethod
    def start_from_local_template_managed(klass, template_file_name):
        template = get_template_path(template_file_name)
        klass.mock_obj.StartFromLocalTemplate(template)

    @classmethod
    def start_from_class(klass, mock_class, params={}):
        mock_server = \
            klass.spawn_server(mock_class.BUS_NAME,
                               mock_class.MAIN_OBJ,
                               mock_class.MAIN_IFACE,
                               mock_class.SYSTEM_BUS,
                               stdout=get_subprocess_stdout())

        bus = klass.get_dbus(system_bus=mock_class.SYSTEM_BUS)
        mock_obj = bus.get_object(mock_class.BUS_NAME, mock_class.MAIN_OBJ)
        mock_class.load(mock_obj, params)

        mocks = (mock_server, mock_obj)
        assert klass.mocks.setdefault(mock_class, mocks) == mocks
        return mocks

    @classmethod
    def init_logind_kvm(klass, session_path):
        session_obj = klass.system_bus_con.get_object('org.freedesktop.login1', session_path)
        session_obj.AddMethod('org.freedesktop.login1.Session',
                              'TakeDevice',
                              'uu', 'hb',
'''
import re

major = args[0]
minor = args[1]

sysfs_uevent_path = '/sys/dev/char/{}:{}/uevent'.format(major, minor)
sysfs_uevent = open(sysfs_uevent_path, 'r')
devname = None
for line in sysfs_uevent.readlines():
    match = re.match('DEVNAME=(.*)', line)
    if match:
        devname = match[1]
        break
sysfs_uevent.close()
if not devname:
    raise dbus.exceptions.DBusException(f'Device file {major}:{minor} doesn\\\'t exist',
                                        major=major, minor=minor)
fd = os.open('/dev/' + devname, os.O_RDWR | os.O_CLOEXEC)
unix_fd = dbus.types.UnixFd(fd)
os.close(fd)
ret = (unix_fd, False)
''')
        session_obj.AddMethods('org.freedesktop.login1.Session', [
            ('ReleaseDevice', 'uu', '', ''),
            ('TakeControl', 'b', '', ''),
        ])

    @classmethod
    def init_logind(klass, enable_kvm):
        logind = klass.start_from_template('logind')

        [p_mock, obj] = logind

        mock_iface = 'org.freedesktop.DBus.Mock'
        obj.AddSeat('seat0', dbus_interface=mock_iface)
        session_path = obj.AddSession('dummy', 'seat0',
                                      dbus.types.UInt32(os.getuid()),
                                      getpass.getuser(),
                                      True,
                                      dbus_interface=mock_iface)

        if enable_kvm:
            klass.init_logind_kvm(session_path)

    def wrap_call(self, args):
        env = {}
        env.update(os.environ)
        env['NO_AT_BRIDGE'] = '1'
        env['GSETTINGS_BACKEND'] = 'memory'

        wrapper = env.get('META_DBUS_RUNNER_WRAPPER')
        if wrapper == 'gdb':
            args = ['gdb', '-ex', 'r', '-ex', 'bt full', '--args'] + args
        elif wrapper:
            args = wrapper.split(' ') + args

        p = subprocess.Popen(args, env=env)
        self.assertEqual(p.wait(), 0)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--kvm', action='store_true', default=False)
    (args, rest) = parser.parse_known_args(sys.argv)

    MutterDBusTestCase.setUpClass(args.kvm)
    test_case = MutterDBusTestCase()
    test_case.assertGreater(len(rest), 1)
    try:
        print('Running test case...', file=sys.stderr)
        test_case.wrap_call(rest[1:])
    finally:
        MutterDBusTestCase.tearDownClass()



# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.

__author__ = 'Marco Trevisan'
__copyright__ = '(c) 2021 Canonical Ltd.'

import dbus
import fcntl
import os
import subprocess

from collections import OrderedDict
from dbusmock import DBusTestCase

BUS_NAME = 'org.gnome.Mutter.TestDBusMocksManager'
MAIN_OBJ = '/org/gnome/Mutter/TestDBusMocksManager'
MAIN_IFACE = 'org.gnome.Mutter.TestDBusMocksManager'
SYSTEM_BUS = True


def load(mock, parameters):
    mock.mocks = OrderedDict()
    DBusTestCase.setUpClass()
    mock.dbus_mock = DBusTestCase()
    mock.dbus_mock.setUp()
    mock.templates_dir = parameters['templates-dir']


def set_nonblock(fd):
    '''Set a file object to non-blocking'''

    flags = fcntl.fcntl(fd, fcntl.F_GETFL)
    fcntl.fcntl(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)


@dbus.service.method(MAIN_IFACE, in_signature='s')
def StartFromTemplate(self, template):
    if template in self.mocks.keys():
        raise KeyError('Template {} already started'.format(template))

    mock_server, mock_obj = self.dbus_mock.spawn_server_template(template, {},
        stdout=subprocess.PIPE)
    set_nonblock(mock_server.stdout)

    self.mocks[template] = (mock_server, mock_obj)


@dbus.service.method(MAIN_IFACE, in_signature='s')
def StartFromLocalTemplate(self, template):
    path = os.path.join(self.templates_dir, template + '.py')
    return self.StartFromTemplate(path)


@dbus.service.method(MAIN_IFACE, in_signature='s')
def StopTemplate(self, template):
    (mock_server, mock_obj) = self.mocks.pop(template)
    mock_server.terminate()
    mock_server.wait()


@dbus.service.method(MAIN_IFACE, in_signature='s')
def StopLocalTemplate(self, template):
    path = os.path.join(self.templates_dir, template + '.py')
    return self.StopTemplate(path)


@dbus.service.method(MAIN_IFACE)
def Cleanup(self):
    for (mock_server, mock_obj) in reversed(self.mocks.values()):
        mock_server.terminate()
        mock_server.wait()

    self.dbus_mock.tearDown()
    DBusTestCase.tearDownClass()


@dbus.service.method(MAIN_IFACE, out_signature='as')
def ListRunningTemplates(self):
    return list(self.mocks.keys())

